package com.seeyon.v3x.edoc.controller;

import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.SystemEnvironment;
import com.seeyon.ctp.common.controller.BaseController;
import com.seeyon.ctp.common.i18n.ResourceUtil;
import com.seeyon.ctp.util.annotation.NeedlessCheckLogin;
import com.seeyon.v3x.edoc.manager.EdocUpgradeManager;

import org.apache.logging.log4j.util.Strings;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;

public class EdocUpgradeControllor extends BaseController {
	
	private EdocUpgradeManager upgradeManager = (EdocUpgradeManager)AppContext.getBean("edocUpgradeManager");
	
	@NeedlessCheckLogin
	public ModelAndView upgrade(HttpServletRequest request, HttpServletResponse response) throws Exception {
		int state = upgradeManager.getUpgradState();
		if(state == upgradeManager.UPGRADE_STATE_DOING){
			super.rendJavaScript(response, ResourceUtil.getString("govdoc.upgrade.doing"));
            return null;
		}else if(state == upgradeManager.UPGRADE_STATE_DONE){
			super.rendJavaScript(response, ResourceUtil.getString("govdoc.upgrade.complete"));
            return null;
		}else if(state == upgradeManager.UPGRADE_STATE_DONE_ERROR){
			super.rendJavaScript(response, ResourceUtil.getString("govdoc.upgrade.error"));
            return null;
		}else if(state == upgradeManager.UPGRADE_STATE_NEEDTODO_NO){
			super.rendJavaScript(response, ResourceUtil.getString("govdoc.upgrade.no"));
            return null;
		}else{
			 try {
				 upgradeManager.setUpgradeState(upgradeManager.UPGRADE_STATE_DOING);
				 HttpSession session = request.getSession(true);
				 session.setMaxInactiveInterval(3600*24*2);
				 String result = upgradeManager.upgrade();
				if ("-1".equals(result)) {
					upgradeManager.setUpgradeState(upgradeManager.UPGRADE_STATE_DONE);
					logger.info("旧版本升级成功!须重启服务方能正常使用!");
					response.setContentType("text/html;charset=UTF-8");
					PrintWriter out = response.getWriter();
					out.println("<script>");
					out.println("alert(\"旧版本升级成功!须重启服务方能正常使用!\");");
					out.println("parent._reflush();");
					out.println("</script>");
					out.flush();
				}
		     } catch (Exception e) {
		    	 upgradeManager.setUpgradeState(upgradeManager.UPGRADE_STATE_DONE_ERROR);
		         super.rendJavaScript(response, ResourceUtil.getString("govdoc.upgrade.fail"));
		     }
		}
        return null;
	}
	
	public ModelAndView download(HttpServletRequest request, HttpServletResponse response) throws Exception {
		OutputStream toClient = null;
		InputStream fis = null;
		try {
			String version = request.getParameter("ver");
			if(Strings.isBlank(version)){
				return null;
			}
			String path = "";
			if("1".equals(version)){ //350版本
				path = SystemEnvironment.getApplicationFolder()+File.separator+"WEB-INF"+File.separator+"jsp"+File.separator+"edoc"
						+File.separator+"upgrade"+File.separator+"edocFormFiles1.jsp";
			}else if("2".equals(version)){ //非350版本
				path = SystemEnvironment.getApplicationFolder()+File.separator+"WEB-INF"+File.separator+"jsp"+File.separator+"edoc"
						+File.separator+"upgrade"+File.separator+"edocFormFiles2.jsp";
			}
            // 以流的形式下载文件。
            fis = new BufferedInputStream(new FileInputStream(path));
            byte[] buffer = new byte[fis.available()];
            fis.read(buffer);
            fis.close();
            // 清空response
            response.reset();
            // 设置response的Header
            response.addHeader("Content-Disposition", "attachment;fileName=edocFormFiles.jsp");
            toClient = new BufferedOutputStream(response.getOutputStream());
            response.setContentType("application/octet-stream");
            toClient.write(buffer);
            toClient.flush();
            toClient.close();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
		finally {
			if (fis != null) {
				try {
					fis.close();
				} catch (Exception e) {

				}
			}
			if (toClient != null) {
				try {
					toClient.close();
				} catch (Exception e) {

				}
			}
		}
		return null;
	}

	public EdocUpgradeManager getUpgradeManager() {
		return upgradeManager;
	}

	public void setUpgradeManager(EdocUpgradeManager upgradeManager) {
		this.upgradeManager = upgradeManager;
	}

}
